"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const instance_1 = require("./instance");
const path = require("path");
const modulesInRootPlugin = require("enhanced-resolve/lib/ModulesInRootPlugin");
const createInnerCallback = require("enhanced-resolve/lib/createInnerCallback");
const getInnerRequest = require("enhanced-resolve/lib/getInnerRequest");
function escapeRegExp(str) {
    return str.replace(/[\-\[\]\/\{\}\(\)\*\+\?\.\\\^\$\|]/g, "\\$&");
}
class TsConfigPathsPlugin {
    constructor(config = {}) {
        this.source = "described-resolve";
        this.target = "resolve";
        const context = config.context || process.cwd();
        const { configFilePath, baseUrl, paths } = instance_1.readConfigFile(context, config);
        console.log(`tsconfig-paths-webpack-plugin: Using ${configFilePath}`);
        this.baseUrl = baseUrl;
        this.absoluteBaseUrl = path.resolve(path.dirname(configFilePath), this.baseUrl || ".");
        this.mappings = [];
        const paths2 = paths || {};
        Object.keys(paths2).forEach(alias => {
            let onlyModule = alias.indexOf("*") === -1;
            let excapedAlias = escapeRegExp(alias);
            let targets = paths2[alias];
            targets.forEach(target => {
                let aliasPattern;
                if (onlyModule) {
                    aliasPattern = new RegExp(`^${excapedAlias}$`);
                }
                else {
                    let withStarCapturing = excapedAlias.replace("\\*", "(.*)");
                    aliasPattern = new RegExp(`^${withStarCapturing}`);
                }
                this.mappings.push({
                    onlyModule,
                    alias,
                    aliasPattern,
                    target: target
                });
            });
        });
    }
    apply(resolver) {
        const { baseUrl, mappings } = this;
        if (baseUrl) {
            resolver.apply(new modulesInRootPlugin("module", this.absoluteBaseUrl, "resolve"));
        }
        mappings.forEach(mapping => {
            // skip "phantom" type references
            if (!this.isTyping(mapping.target)) {
                resolver.plugin(this.source, this.createPlugin(resolver, mapping));
            }
        });
    }
    isTyping(target) {
        return target.indexOf("@types") !== -1 || target.indexOf(".d.ts") !== -1;
    }
    createPlugin(resolver, mapping) {
        return (request, callback) => {
            const innerRequest = getInnerRequest(resolver, request);
            if (!innerRequest) {
                return callback();
            }
            const match = innerRequest.match(mapping.aliasPattern);
            if (!match) {
                return callback();
            }
            let newRequestStr = mapping.target;
            if (!mapping.onlyModule) {
                newRequestStr = newRequestStr.replace("*", match[1]);
            }
            if (newRequestStr[0] === ".") {
                newRequestStr = path.resolve(this.absoluteBaseUrl, newRequestStr);
            }
            // const newRequest = _.extend({}, request, {
            //   request: newRequestStr
            // });
            const newRequest = Object.assign({}, request, { request: newRequestStr });
            return resolver.doResolve(this.target, newRequest, "aliased with mapping '" +
                innerRequest +
                "': '" +
                mapping.alias +
                "' to '" +
                newRequestStr +
                "'", createInnerCallback(function (err, result) {
                if (arguments.length > 0) {
                    return callback(err, result);
                }
                // don't allow other aliasing or raw request
                callback(null, null);
            }, callback));
        };
    }
}
exports.TsConfigPathsPlugin = TsConfigPathsPlugin;
